
rm(list=ls())

# setwd("~/Dropbox/chun-stuff/orm/quasi-reproducible")

## Loading simulated data on ART initiators and baseline covariates.  This was created in 
##  the code for the long-term mortality study (Carriquiry, et al., 2015)
load("d-second-chunk.Rda")

## Loading simulated repeated CD4 measurements
d.cd4<-read.csv("lab_cd4_sim2.csv")

## Removing records with missing or undetectable baseline viral load and missing CD4
d<-d[!is.na(d$rna1),]
d<-d[d$rna1>0,]
d<-d[!is.na(d$nadir.cd4.prehaart),]

##  Merging 6-month CD4 measurements with baseline data
d.ids<-d[,c("id","init.start.date")]
ids<-d.ids$id
d.cd4.new<-d.cd4[d.cd4$patient %in% ids,]
d.cd4.new1<-d.cd4.new[d.cd4.new$time<300 & d.cd4.new$time>60,]
d.cd4.new2<-d.cd4.new1[!duplicated(d.cd4.new1$patient),]
d.cd4.new2$post.cd4<-d.cd4.new2$cd4_v
d.cd4.new3<-d.cd4.new2[,c("patient","post.cd4")]

d1<-merge(d.cd4.new3, d, by.x="patient", by.y="id", all.x=TRUE)

##  Re-naming variables and creating a new dataset that mimics the one used in the CPM CD4 analysis
d1$nadir.cd4<-d1$nadir.cd4.prehaart
d1$init.year<-d1$year.init
d1$pre.rna<-d1$rna1

d<-d1[,c("site","male","age","init.class","route","nadir.cd4","init.year","pre.rna","preARTAIDS.clinical","post.cd4")]


#######################################################################################
##########################  Code for CPMs    ##########################################
#######################################################################################


################################### FUNCTIONS ################

#### function to estimate conditional mean and its standard error for orm model
mean.orm <- function(mod, new.data, se=TRUE){
  if(is.null(mod$yunique)) {
    stop("Need to set x=TRUE and y=TRUE for orm") 
  } else{
    order.y <- mod$yunique
    n.alpha <- length(order.y)-1
    xb <- as.matrix(new.data)%*%matrix(coef(mod)[colnames(new.data)])
    m.alpha <- mod$coef[1:n.alpha]
    lb <- t(outer(m.alpha, xb, "+")[,,1])
    m.s <- mod$trans$cumprob(lb)
    m.f <- t(apply(m.s, 1, FUN=function(x) c(1,x[1:n.alpha]) - c(x[1:n.alpha], 0)))
    m.mean <- apply(m.f, 1, FUN=function(x) sum(x*order.y))
    
    if(se){
      if(mod$family=="logistic") mod$trans$deriv <- function(x) exp(-x)/(1+exp(-x))^2
      
      dmean.dalpha <- t(apply(mod$trans$deriv(lb), 1, FUN=function(x) x*(order.y[2:length(order.y)] - order.y[1:n.alpha])))
      dmean.dbeta <- apply(dmean.dalpha, 1, sum)*as.matrix(new.data)
      dmean.dtheta <- cbind(dmean.dalpha, dmean.dbeta)   
      mean.var <-diag(dmean.dtheta%*%solve(mod$info.matrix)%*%t(dmean.dtheta))
      mean.se <- sqrt(mean.var)   
      result <- cbind(m.mean, mean.se)
      ci <- t(apply(result, 1, FUN=function(x) c(x[1]- qnorm(0.975)*x[2], x[1]+ qnorm(0.975)*x[2])))
      result <- cbind(result, ci)
      colnames(result) <- c("est", "se", "lb", "ub")
    } else{
      result <- matrix(m.mean)
      colnames(result) <- c("est")
    }
    
    
    return(result)
    
    
  } 
  
}


#### function to estimate conditional quantiles and confidence intervals for orm model
quantile.orm <- function(mod, new.data, probs=0.5, se=TRUE){
  
  quantile <- matrix(NA, nrow=dim(new.data)[1], ncol=length(probs))
  order.y <- mod$yunique
  #n.alpha <- length(order.y)-1
  xb <- as.matrix(new.data)%*%matrix(coef(mod)[colnames(new.data)])
  alpha <- mod$coef[1:(length(unique(order.y))-1)]
  lb <- t(outer(alpha, xb, "+")[,,1])
  m.cdf <- 1- mod$trans$cumprob(lb)
  m.cdf <- cbind(0, m.cdf, 1)
  for(i in 1: length(probs)){
    try({
      index.1 <- apply(m.cdf, 1, FUN=function(x){ max(which(x<=probs[i]))[1]} )
      index.2 <- apply(m.cdf, 1, FUN=function(x){ min(which(x>=probs[i]))[1]} )
      
      index.y1 <- ifelse(index.1>length(order.y), Inf, order.y[index.1])
      index.y2 <- ifelse(index.2>length(order.y),Inf,order.y[index.2])
      
      index.y1.cdf <- ifelse(index.1==0, 0, m.cdf[cbind(1:dim(new.data)[1], index.1)])
      
      index.y2.cdf <- ifelse(index.2>length(order.y), 1, m.cdf[cbind(1:dim(new.data)[1], index.2)])
      
      
      
      
      quantile[,i] <- ifelse(index.1==index.2, index.y1, 
                             (index.y2-index.y1)/(index.y2.cdf - index.y1.cdf)*(probs[i]-index.y1.cdf) + index.y1) 
      quantile[, i] <- ifelse(is.infinite(quantile[,i]), max(order.y), quantile[, i])
    })
    
  }
  result <- quantile
  
  if(se){
    if(mod$family=="logistic") mod$trans$deriv <- function(x) exp(-x)/(1+exp(-x))^2
    
    quantile.lb <- quantile.ub <- matrix(NA, nrow=dim(new.data)[1], ncol=length(probs))
    lb.se <- matrix(NA, ncol=dim(lb)[2], nrow=dim(new.data)[1])
    var <- as.matrix(solve(mod$info.matrix))
    
    for(i in 1:dim(lb)[2]){
      var.i <- var[c(i, which(names(coef(mod)) %in% colnames(new.data))), 
                   c(i, which(names(coef(mod)) %in% colnames(new.data)))]
      
      dcdf.dtheta <- cbind(-mod$trans$deriv(lb[,i]),  
                           -mod$trans$deriv(lb[,i])*as.matrix(new.data) )
      dlb.dtheta <- as.matrix(cbind(1, new.data))
      lb.se[,i] <- sqrt(diag(dlb.dtheta%*%var.i%*% t(dlb.dtheta)))
    }
    
    ci.lb <- sapply(1:dim(lb)[2], FUN=function(i) { 1- mod$trans$cumprob(lb[, i] +qnorm(0.975)*lb.se[, i])})
    ci.ub <- sapply(1:dim(lb)[2], FUN=function(i) { 1- mod$trans$cumprob(lb[, i] -qnorm(0.975)*lb.se[, i])})
    ci.lb <- matrix(ci.lb, nrow=dim(new.data)[1])
    ci.ub <- matrix(ci.ub, nrow=dim(new.data)[1])
    
    ci.lb <- cbind(0, ci.lb, 1)
    ci.ub <- cbind(0, ci.ub, 1)
    
    for(i in 1: length(probs)){
      try({
        index.1 <- apply(ci.lb, 1, FUN=function(x){ max(which(x<=probs[i]))[1]} )
        index.2 <- apply(ci.lb, 1, FUN=function(x){ min(which(x>=probs[i]))[1]} )
        
        index.y1 <- ifelse(index.1>length(order.y), Inf, order.y[index.1])
        index.y2 <- ifelse(index.2>length(order.y),Inf,order.y[index.2])
        
        index.y1.cdf <- ifelse(index.1==0, 0, ci.lb[cbind(1:dim(new.data)[1], index.1)])
        
        index.y2.cdf <- ifelse(index.2>length(order.y), 1, ci.lb[cbind(1:dim(new.data)[1], index.2)])
        
        
        quantile.lb[,i] <- ifelse(index.1==index.2, index.y1, 
                                  (index.y2-index.y1)/(index.y2.cdf - index.y1.cdf)*(probs[i]-index.y1.cdf) + index.y1) 
        quantile.lb[, i] <- ifelse(is.infinite(quantile.lb[,i]), max(order.y), quantile.lb[, i])
        
        index.1 <- apply(ci.ub, 1, FUN=function(x){ max(which(x<=probs[i]))[1]} )
        index.2 <- apply(ci.ub, 1, FUN=function(x){ min(which(x>=probs[i]))[1]} )
        
        index.y1 <- ifelse(index.1>length(order.y), Inf, order.y[index.1])
        index.y2 <- ifelse(index.2>length(order.y),Inf,order.y[index.2])
        
        index.y1.cdf <- ifelse(index.1==0, 0, ci.ub[cbind(1:dim(new.data)[1], index.1)])
        
        index.y2.cdf <- ifelse(index.2>length(order.y), 1, ci.ub[cbind(1:dim(new.data)[1], index.2)])
        
        
        quantile.ub[,i] <- ifelse(index.1==index.2, index.y1, 
                                  (index.y2-index.y1)/(index.y2.cdf - index.y1.cdf)*(probs[i]-index.y1.cdf) + index.y1) 
        quantile.ub[, i] <- ifelse(is.infinite(quantile.ub[,i]), max(order.y), quantile.ub[, i])
        
        
      })
      
    }
    
    result <- list(quantile=quantile,
                   lb=quantile.ub,
                   ub=quantile.lb)
    
    
    
  }
  
  
  
  return(result)
  
}


#### function to estimate conditional CDF and its standard error for orm models
cdf.orm <- function(mod, new.data, at.y=0,se=TRUE){
  if(is.null(mod$yunique)) {
    stop("Need to set x=TRUE and y=TRUE for orm") 
  } else{
    order.y <- mod$yunique
    xb <- as.matrix(new.data)%*%matrix(coef(mod)[colnames(new.data)])
    
    index <- sapply(at.y, FUN=function(x) {if(x<min(order.y)[1]) result <- Inf 
    else if (x==min(order.y)[1]) result <- 1
    else if(x >= max(order.y)[1]) result <- -Inf
    else which(order.y>=x)[1]-1})
    
    m.alpha <- mod$coef[index]
    m.alpha <- ifelse(is.infinite(index), index, m.alpha)
    if(length(at.y)==1){
      lb <- as.matrix(outer(m.alpha, xb, "+")[,,1])
    } else lb <- t(outer(m.alpha, xb, "+")[,,1])
    m.cdf <- 1- mod$trans$cumprob(lb)
    
    
    if(se){
      if(mod$family=="logistic") mod$trans$deriv <- function(x) exp(-x)/(1+exp(-x))^2
      cdf.se <- matrix(NA, ncol=length(at.y), nrow=dim(new.data)[1])
      lb.se <- matrix(NA, ncol=length(at.y), nrow=dim(new.data)[1])
      
      var <- as.matrix(solve(mod$info.matrix))
      
      for(i in 1:length(at.y)) {
        
        var.i <- var[c(index[i], which(names(coef(mod)) %in% colnames(new.data))), 
                     c(index[i], which(names(coef(mod)) %in% colnames(new.data)))]
        dcdf.dtheta <- cbind(-mod$trans$deriv(lb[,i]),  
                             -mod$trans$deriv(lb[,i])*as.matrix(new.data) )
        dlb.dtheta <- as.matrix(cbind(1, new.data))
        cdf.se[,i] <- sqrt(diag(dcdf.dtheta %*% var.i%*% t(dcdf.dtheta)))
        lb.se[, i] <- sqrt(diag(dlb.dtheta%*%var.i%*% t(dlb.dtheta)))
        
      }
      ci.lb <- sapply(1:length(at.y), FUN=function(i) { 1- mod$trans$cumprob(lb[, i] +qnorm(0.975)*lb.se[, i])})
      ci.ub <- sapply(1:length(at.y), FUN=function(i) { 1- mod$trans$cumprob(lb[, i] -qnorm(0.975)*lb.se[, i])})
      
      
      result <- list(est=m.cdf,
                     se=cdf.se,
                     lb=ci.lb,
                     ub=ci.ub)
    } else{
      result <- list(est=m.cdf)
    }
    
    
    return(result)
    
  } 
  
}


##### function for calculate probility-scale residuals. 
########This has also been implemented as presid() in "PResiduals" package
presid.orm <- function(object){
  k <- object$non.slopes
  L <- object$linear.predictors
  trans <- object$trans
  cumprob <- if(length(trans)) trans$cumprob else plogis
  if(length(L)==0) stop('you did not use linear.predictors=TRUE for the fit')
  if(length(Y <- object$y) == 0) stop("you did not specify y=TRUE in the fit")
  if(!is.factor(Y)) Y <- factor(Y)
  Y <- unclass(Y) - 1
  cof <- object$coefficients
  if(length(X <- object$x)==0)
    stop("you did not use x=TRUE in the fit")
  
  
  N <- length(Y)
  px <- 1 - cumprob(outer(cof[1:k],
                          as.vector(X %*% cof[- (1:k)]), "+"))
  low.x = rbind(0, px)[cbind(Y + 1L, 1:N)]
  hi.x  = 1 - rbind(px, 1)[cbind(Y + 1L, 1:N)]
  return(low.x - hi.x)
}

#### Or use presid(), see details in the help documents
library(PResiduals)
library(rms)
help(presid)

############### CD4 ############################

###### cumulative probability models with different link functions #####
dd<-datadist(d); options(datadist='dd')
orm.logit <- orm(post.cd4~ site + male + rcs(age) + init.class + route + 
                   rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
                   preARTAIDS.clinical, data=d, x=TRUE, y=TRUE)

orm.probit <- orm(post.cd4~ site + male + rcs(age) + init.class + route + 
                    rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
                    preARTAIDS.clinical, data=d, x=TRUE, y=TRUE, family=probit)

orm.cloglog <- orm(post.cd4~ site + male + rcs(age) + init.class + route + 
                     rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
                     preARTAIDS.clinical, data=d, x=TRUE, y=TRUE, family=cloglog)

orm.loglog <- orm(post.cd4~ site + male + rcs(age) + init.class + route + 
                    rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
                    preARTAIDS.clinical, data=d, x=TRUE, y=TRUE, family=loglog)### loglog in orm correspoonds to cloglog in conventional cumulative link models

orm.cloglog <- orm.loglog


##### probability-scale residuals
# likelihoods (effectively compare the AIC since number of parameters are the same)
log.L <- c(-orm.probit$deviance[2], -orm.logit$deviance[2], -orm.cloglog$deviance[2])/2
log.L <- c(-orm.probit$deviance[2], -orm.logit$deviance[2], -orm.cloglog$deviance[2])/2

names(log.L) <- c("probit", "logit", "cloglog")
log.L
# 
# probit     logit   cloglog 
# -28796.42 -28709.87 -29185.60 

presid.logit <- presid.orm(orm.logit)

presid.probit <- presid.orm(orm.probit)

presid.cloglog <- presid.orm(orm.cloglog)

##### Box-Cox transformation

library(MASS)

boxcox.lambda<-boxcox(post.cd4 ~ site + male + rcs(age) + init.class + route + 
                        rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
                        preARTAIDS.clinical, data=d)

lambda <- boxcox.lambda$x[which(boxcox.lambda$y==max(boxcox.lambda$y))]
box.cox.trans <- function(x,lambda){
  if(lambda==0){
    x <- log(x)
  }else{
    x <- (x^lambda-1)/lambda
  }
  
  return(x)
}


box.cox.trans.inverse <- function(y, lambda){
  if(lambda==0){
    x <- exp(y)
  }else{
    x <- (lambda*y+1)^{1/lambda}
  }
  return(x)
}

d$post.cd4.boxcox <- box.cox.trans(d$post.cd4, lambda)
lm.boxcox <-  ols(post.cd4.boxcox ~ site + male + rcs(age) + init.class + route + 
                    rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
                    preARTAIDS.clinical, data=d, y=TRUE, x=TRUE)

presid.boxcox <- presid(lm.boxcox)


###### Figure 12

postscript("fig13-app_1.eps", height=6, width=8.5,  horizontal = FALSE, onefile = FALSE, paper = "special")
par(mfrow=c(3,4))
par(mar=c(2.5,2.7,0.5,0.2), mgp = c(1.5, 0.5, 0), oma=c(0.5,3,2,0.5))


alpha.probit <- -orm.probit$coefficients[1:(length(orm.probit$yunique)-1)]
plot(orm.probit$yunique, c(alpha.probit, Inf), type="s", ylab=expression(paste(hat(alpha), "(y)")), xlab="CD4")


alpha.logit <- -orm.logit$coefficients[1:(length(orm.logit$yunique)-1)]
plot(orm.logit$yunique, c(alpha.logit, Inf), type="s", ylab=expression(paste(hat(alpha), "(y)")), xlab="CD4")

alpha.cloglog <- -orm.cloglog$coefficients[1:(length(orm.cloglog$yunique)-1)]
plot(orm.cloglog$yunique, c(alpha.cloglog, Inf), type="s", ylab=expression(paste(hat(alpha), "(y)")), xlab="CD4")

plot(c(min(d$post.cd4):max(d$post.cd4)), box.cox.trans(c(min(d$post.cd4):max(d$post.cd4)), lambda), type="n",
     xlab="CD4", ylab="Estimated transformation")
lines(c(min(d$post.cd4):max(d$post.cd4)), box.cox.trans(c(min(d$post.cd4):max(d$post.cd4)), lambda), lty=1)

order.presid.probit <- presid.probit[order(presid.probit)] 
order.presid.logit <- presid.logit[order(presid.logit)] 
order.presid.cloglog <- presid.cloglog[order(presid.cloglog)] 
order.presid.boxcox <- presid.boxcox[order(presid.boxcox)]
plot( qunif( (1:length(presid.logit))/length(presid.logit), -1, 1),
      order.presid.probit,
      cex=0.1, xlab="Quantiles of Uniform Distribution", ylab="Quantiles of PSR", main="")


#qqline(order.presid.probit, distribution=function(x) qunif(x,-1,1), col=2)
abline(a=0, b=1, col=2)

plot(qunif( (1:length(presid.logit))/length(presid.logit), -1, 1), 
     order.presid.logit, cex=0.1, xlab="Quantiles of Uniform Distribution", ylab="Quantiles of PSR", main="")
abline(a=0, b=1, col=2)
#qqline(order.presid.logit, distribution=function(x) qunif(x,-1,1), col=2)

plot( qunif( (1:length(presid.logit))/length(presid.logit), -1, 1), 
      order.presid.cloglog,cex=0.1, xlab="Quantiles of Uniform Distribution", ylab="Quantiles of PSR", main="")
abline(a=0, b=1, col=2)

#qqline(order.presid.cloglog, distribution=function(x) qunif(x,-1,1), col=2)

plot( qunif( (1:length(presid.logit))/length(presid.logit), -1, 1), 
      order.presid.boxcox,cex=0.1, xlab="Quantiles of Uniform Distribution", ylab="Quantiles of PSR", main="")
abline(a=0, b=1, col=2)

orm.omer.logit <- -orm.logit$coefficients[sapply(d$post.cd4, FUN=function(x) which(orm.logit$yunique==x))] - (predict(orm.logit, type="lp", kint=1) - orm.logit$coefficients[1])
orm.omer.logit <- orm.omer.logit[-which(d$post.cd4==max(d$post.cd4))]

orm.omer.probit <- -orm.probit$coefficients[sapply(d$post.cd4, FUN=function(x) which(orm.probit$yunique==x))] - (predict(orm.probit, type="lp", kint=1) - orm.probit$coefficients[1])
orm.omer.probit <- orm.omer.probit[-which(d$post.cd4==max(d$post.cd4))]

orm.omer.cloglog <- -orm.cloglog$coefficients[sapply(d$post.cd4, FUN=function(x) which(orm.cloglog$yunique==x))] - (predict(orm.cloglog, type="lp", kint=1) - orm.cloglog$coefficients[1])
orm.omer.cloglog <- orm.omer.cloglog[-which(d$post.cd4==max(d$post.cd4))]


order.orm.omer.probit <- orm.omer.probit[order(orm.omer.probit)] 
order.orm.omer.logit <- orm.omer.logit[order(orm.omer.logit)] 
order.orm.omer.cloglog <- orm.omer.cloglog[order(orm.omer.cloglog)] 

plot( qnorm( (1:length(orm.omer.logit))/length(orm.omer.logit)),
      order.orm.omer.probit,
      cex=0.25,, xlab="Quantiles of Normal Distribution", ylab="Quantiles of OMER ", main="")

abline(a=0, b=1, col=2)

plot(qlogis( (1:length(orm.omer.logit))/length(orm.omer.logit)), 
     order.orm.omer.logit, cex=0.25, xlab="Quantiles of Logistic Distribution", ylab="Quantiles of OMER", main="")
abline(a=0, b=1, col=2)


qGumbelMax <- function(p, mu=0, sigma=1){
  x <- mu + -sigma*(log(-log(p)))
  
}

qGumbelMin <- function(p, mu=0, sigma=1){
  x <- mu + sigma*(log(-log(1-p)))
  
}


plot( qGumbelMin((1:length(orm.omer.logit))/length(orm.omer.logit), 0, 1), 
      order.orm.omer.cloglog, cex=0.25, xlab="Quantiles of Extreme Value (I) ", ylab="Quantiles of OMER", main="")
abline(a=0, b=1, col=2)

qqnorm(lm.boxcox$residuals/sd(lm.boxcox$residuals), cex=0.25,  main="", xlab="Quantiles of Normal Distribution", ylab="Quantiles of OMER")
abline(a=0, b=1, col=2)

mtext( "(a)", side=2, outer=TRUE, at=1, cex=1, las=1)
mtext( "(b)", side=2, outer=TRUE, at=2/3, cex=1, las=1)
mtext( "(c)", side=2, outer=TRUE, at=1/3, cex=1, las=1)

mtext("probit", side=3, at=1/8, outer=TRUE, cex=1)
mtext("logit", side=3, at=3/8, outer=TRUE, cex=1)
mtext("cloglog", side=3, at=5/8, outer=TRUE, cex=1)
mtext("Box-Cox", side=3, at=7/8, outer=TRUE, cex=1)

dev.off()

##### Figure 13: illustrate residual-by-predictor plot ####
#### not include nadir cd4
orm.logit.2 <- orm(post.cd4~ site + male  + rcs(age)+init.class + route + 
                     rcs(init.year)+ rcs(log(pre.rna))+
                     preARTAIDS.clinical, data=d, x=TRUE, y=TRUE)
presid.logit.2 <- presid.orm(orm.logit.2)

postscript("fig14-app_2.eps", height=6, width=6,  horizontal = FALSE, onefile = FALSE, paper = "special")
par(mfcol=c(2,2))
par(mar=c(2.5,2.5,0.5,0.5), mgp = c(1.5, 0.5, 0), oma=c(0.5,0.5,2,0.5))

plot(sqrt(d$nadir.cd4), presid.logit.2, cex=0.1, type="n",
     xlab="square root of nadir CD4", ylab="PSR", main="")
points(sqrt(d$nadir.cd4), presid.logit.2,pch=1,cex=.5,col=gray(.6))
lines(supsmu(sqrt(d$nadir.cd4), presid.logit.2, bass=1),col=1,lwd=2, cex=1)
abline(h=0,lty=2)

orm.omer.logit.2 <- -orm.logit.2$coefficients[sapply(d$post.cd4, FUN=function(x) which(orm.logit.2$yunique==x))] - (predict(orm.logit.2, type="lp", kint=1) - orm.logit.2$coefficients[1])
orm.omer.logit.2 <- orm.omer.logit.2[-which(d$post.cd4==max(d$post.cd4))]
plot(sqrt(d$nadir.cd4[-which(d$post.cd4==max(d$post.cd4))]), orm.omer.logit.2, cex=0.1, type="n", xlab="square root of nadir CD4", ylab="OMER")
points(sqrt(d$nadir.cd4[-which(d$post.cd4==max(d$post.cd4))]), orm.omer.logit.2,pch=1,cex=.5,col=gray(.6))
lines(supsmu(sqrt(d$nadir.cd4[-which(d$post.cd4==max(d$post.cd4))]), orm.omer.logit.2, bass=1),col=1,lwd=2, cex=1)
abline(h=0,lty=2)

plot(d$nadir.cd4, presid.logit, cex=0.1, type="n",
     xlab="square root of nadir CD4", ylab="PSR", main="")
points(d$nadir.cd4,presid.logit,pch=1,cex=.5,col=gray(.6))
lines(supsmu(d$nadir.cd4, presid.logit, bass=1),col=1,lwd=2, cex=1)
abline(h=0,lty=2)

plot(d$nadir.cd4[-which(d$post.cd4==max(d$post.cd4))], orm.omer.logit, cex=0.1, type="n", xlab="square root of nadir CD4", ylab="OMER ")
points(d$nadir.cd4[-which(d$post.cd4==max(d$post.cd4))], orm.omer.logit,pch=1,cex=.5,col=gray(.6))
lines(supsmu(d$nadir.cd4[-which(d$post.cd4==max(d$post.cd4))], orm.omer.logit, bass=1),col=1,lwd=2, cex=1)
abline(h=0,lty=2)

mtext("not include nadir CD4", side=3, at=1/4, outer=TRUE, cex=1)
mtext("include nadir CD4", side=3, at=3/4, outer=TRUE, cex=1)
dev.off()

##### Figure 14: estimate conditional distribution

##### conditional mean: orm with logit vs Box-Cox
new.age <- (180:820)/10
new.data <- matrix(NA, ncol=dim(orm.logit$x)[2], nrow=length(new.age))

colnames(new.data) <- colnames(orm.logit$x)
for(i in 1:5) new.data[,i] <- rep(orm.logit$x[which(d$site=="brazil")[1], i], length(new.age))

new.data[, "male=1"] <- 1
new.data[, 7:10] <-  rcspline.eval(new.age, knots=orm.logit$Design$parms$age, inclx=TRUE)

for(i in 11:13) new.data[,i] <- rep(orm.logit$x[which(d$init.class=="NNRTI")[1], i], length(new.age))
for(i in 14:17) new.data[,i] <- rep(orm.logit$x[which(d$route=="homo/bisexual")[1], i], length(new.age))

for(i in 18:21) new.data[,i] <- rep(orm.logit$x[which(d$nadir.cd4==167)[1], i], length(new.age))
for(i in 22:25) new.data[,i] <- rep(orm.logit$x[which(d$init.year==2010)[1], i], length(new.age))
new.data[,26:29] <- matrix(rcspline.eval(log(91728),  knots=orm.logit$Design$parms$pre.rna, inclx=TRUE),nrow=length(new.age), ncol=4, byrow=TRUE)
for(i in 30:31) new.data[,i] <- rep(orm.logit$x[which(d$preARTAIDS.clinical=="not AIDS")[1], i], length(new.age)) 



##### conditional mean as a function of age, fix other covariates at median (for continuous variables) or mode (for discrete variables)
orm.logit <- orm(post.cd4~ site + male + rcs(age) + init.class + route + 
                   rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
                   preARTAIDS.clinical, data=d, x=TRUE, y=TRUE)

est.mean.orm.logit <- mean.orm(orm.logit,new.data,se=TRUE )

new.data.ols <- data.frame(site="brazil", male=1, age=new.age, 
                           init.class="NNRTI", route="homo/bisexual", 
                           nadir.cd4=167, init.year=2010, pre.rna=91728,
                           preARTAIDS.clinical="not AIDS")

est.lm.boxcox.t <- predict(lm.boxcox, new.data.ols, se.fit=TRUE, conf.int=0.95)


est.mean.boxcox <- cbind(box.cox.trans.inverse(est.lm.boxcox.t$linear.predictors, lambda),
                         box.cox.trans.inverse(est.lm.boxcox.t$lower, lambda),
                         box.cox.trans.inverse(est.lm.boxcox.t$upper, lambda))

##### conditional median: orm logit vs median regression

est.median.orm.logit <- quantile.orm(mod=orm.logit,new.data=new.data, probs=0.5,se=TRUE )
library(quantreg)

rq.mod <- rq(post.cd4~ site + male + rcs(age) + init.class + route + 
               rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
               preARTAIDS.clinical, data=d)

X <-  cbind(1, new.data)
pred <-drop(X%*%rq.mod$coef)
V <- summary(rq.mod, cov = TRUE)
df <- V$rdf
tfrac <- qt((1 - 0.95)/2, df)
sdpred <- sqrt(diag(X %*% V$cov %*% t(X)))

est.median.rq <- cbind(pred, pred + tfrac * sdpred %o% 
                         c(1, -1))
colnames(est.median.rq) <- c("fit", "lower", "higher")


est.cdf.orm.logit <- cdf.orm(mod=orm.logit, new.data=new.data, at.y=500)
est.cdf.orm.logit2 <- cdf.orm(mod=orm.logit, new.data=new.data, at.y=350)
#est.cdf.orm.logit3 <- cdf.orm(mod=orm.logit, new.data=new.data, at.y=150)

d$post.cd4.350 <- ifelse(d$post.cd4>350, 1, 0)
d$post.cd4.500 <- ifelse(d$post.cd4>500, 1, 0)
#d$post.cd4.150 <- ifelse(d$post.cd4>150, 1, 0)

##### logistic regression models
lrm.500 <- lrm(post.cd4.500 ~ site + male + rcs(age) + init.class + route + 
                 rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
                 preARTAIDS.clinical, data=d, x=TRUE)
test <- predict(lrm.500, new.data.ols, se.fit=TRUE)
est.cdf.glm.logit <- cbind(plogis(test$linear.predictors), plogis(with(test, linear.predictors + 1.96*cbind(-se.fit,se.fit))))

lrm.350 <- lrm(post.cd4.350 ~ site + male + rcs(age) + init.class + route + 
                 rcs(sqrt(nadir.cd4)) +rcs(init.year)+ rcs(log(pre.rna))+
                 preARTAIDS.clinical, data=d, x=TRUE)
test <- predict(lrm.350, new.data.ols, se.fit=TRUE)
est.cdf.glm.logit2 <- cbind(plogis(test$linear.predictors), plogis(with(test, linear.predictors + 1.96*cbind(-se.fit,se.fit))))




postscript("fig15-app_3.eps", height=4.5, width=8,  horizontal = FALSE, onefile = FALSE, paper = "special")
par(mfrow=c(2,4))
par(mar=c(2.5,2.5,0.5,0.5), mgp = c(1.5, 0.5, 0), oma=c(0.5,3,2,0.5))
plot(new.age, est.mean.orm.logit[,1], cex= 0.01, ylim=c(200, 410), ylab="CD4", xlab="age", type="n", main="")


polygon(c(new.age, rev(new.age)), c(est.mean.boxcox[,2], rev(est.mean.boxcox[,3])), col =colors()[560], border = NA)
polygon(c(new.age, rev(new.age)), c(est.mean.orm.logit[,3], rev(est.mean.orm.logit[,4])), col = "grey", border = NA)
lines(new.age, est.mean.orm.logit[,1], cex=0.01)
lines(new.age, est.mean.orm.logit[,3], cex=0.01, lty=2)
lines(new.age, est.mean.orm.logit[,4], cex=0.01, lty=2)

lines(new.age, est.mean.boxcox[,1], cex=0.01, col=2)
lines(new.age, est.mean.boxcox[,2], cex=0.01, col=2, lty=2)
lines(new.age, est.mean.boxcox[,3], cex=0.01, col=2, lty=2)

legend("top", c("orm (logit)", "Box-Cox" ), col=c(1,2), lty=c(1,1),bty="n")



plot(new.age, est.median.orm.logit$quantile, cex=0.01, ylim=c(200, 410), ylab="CD4", xlab="age", type="n")


polygon(c(new.age, rev(new.age)), c(est.median.rq[,2], rev(est.median.rq[,3])),col =colors()[560], border = NA)
polygon(c(new.age, rev(new.age)), c(est.median.orm.logit$lb, rev(est.median.orm.logit$ub)), col = 'grey', border = NA)
lines(new.age, est.median.orm.logit$quantile, cex=0.01)
lines(new.age, est.median.orm.logit$lb, cex=0.01, lty=2)
lines(new.age, est.median.orm.logit$ub, cex=0.01, lty=2)

lines(new.age, est.median.rq[,1], cex=0.01, col=2)
lines(new.age, est.median.rq[,2], cex=0.01, col=2, lty=2)
lines(new.age, est.median.rq[,3], cex=0.01, col=2, lty=2)

legend("top", c("orm (logit)", "Median regression"), col=c(1,2), lty=c(1,1),bty="n")

plot(new.age,1-est.cdf.orm.logit2$est[,1], cex=0.01, xlab="age", ylab="Probability", ylim=c(0,0.6), typ="n")
polygon(c(new.age, rev(new.age)),  c(est.cdf.glm.logit2[,2], rev(est.cdf.glm.logit2[,3])), col =colors()[560] , border = NA)
polygon(c(new.age, rev(new.age)), c(1-est.cdf.orm.logit2$ub[,1], rev(1-est.cdf.orm.logit2$lb[,1])), col = 'grey', border = NA)


lines(new.age, 1-est.cdf.orm.logit2$est[,1], cex=0.01)
lines(new.age, 1-est.cdf.orm.logit2$ub[,1], cex=0.01, lty=2)
lines(new.age, 1-est.cdf.orm.logit2$lb[,1], cex=0.01, lty=2)

lines(new.age, est.cdf.glm.logit2[,1], cex=0.01, col=2)
lines(new.age, est.cdf.glm.logit2[,2], cex=0.01, col=2, lty=2)
lines(new.age, est.cdf.glm.logit2[,3], cex=0.01, col=2, lty=2)

legend("top", c("orm (logit)", "logistic regression"), col=c(1,2), lty=c(1,1),bty="n")


plot(new.age,1-est.cdf.orm.logit$est[,1], cex=0.01, xlab="age", ylab="Probability", ylim=c(0,0.6), typ="n")
polygon(c(new.age, rev(new.age)),  c(est.cdf.glm.logit[,2], rev(est.cdf.glm.logit[,3])), col = colors()[560], border = NA)
polygon(c(new.age, rev(new.age)), c(1-est.cdf.orm.logit$ub[,1], rev(1-est.cdf.orm.logit$lb[,1])), col = 'grey', border = NA)

lines(new.age, 1-est.cdf.orm.logit$est[,1], cex=0.01)
lines(new.age, 1-est.cdf.orm.logit$ub[,1], cex=0.01, lty=2)
lines(new.age, 1-est.cdf.orm.logit$lb[,1], cex=0.01, lty=2)
lines(new.age, est.cdf.glm.logit[,1], cex=0.01, col=2)
lines(new.age, est.cdf.glm.logit[,2], cex=0.01, col=2, lty=2)
lines(new.age, est.cdf.glm.logit[,3], cex=0.01, col=2, lty=2)

legend("top", c("orm (logit)", "logistic regression"), col=c(1,2), lty=c(1,1),bty="n")

mtext("Mean", side=3, at=1/8, outer=TRUE, cex=1)
mtext("Median", side=3, at=3/8, outer=TRUE, cex=1)
mtext("P[CD4>350|Z]", side=3, at=5/8, outer=TRUE, cex=1)
mtext("P[CD4>500|Z]", side=3, at=7/8, outer=TRUE, cex=1)




####### by treatment class
new.data2 <- new.data[1:4,]

new.data2[,7:10] <- matrix(rcspline.eval(35,  knots=orm.logit$Design$parms$age, inclx=TRUE),nrow=4, ncol=4, byrow=TRUE)

new.data2[1, 11:13] <- orm.logit$x[which(d$init.class=="Boosted PI")[1], 11:13]

new.data2[2, 11:13] <- orm.logit$x[which(d$init.class=="NNRTI")[1], 11:13]
new.data2[3, 11:13] <- orm.logit$x[which(d$init.class=="Unboosted PI")[1], 11:13]
new.data2[4, 11:13] <- orm.logit$x[which(d$init.class=="Other")[1], 11:13]


est.mean.orm.logit <- mean.orm(orm.logit, new.data2,se=TRUE )

new.data.ols <- data.frame(site="brazil", male=1, age=35, 
                           init.class=c("Boosted PI","NNRTI","Unboosted PI","Other"), route="homo/bisexual", 
                           nadir.cd4=167, init.year=2010, pre.rna=91728,
                           preARTAIDS.clinical="not AIDS")



est.lm.boxcox.t <- predict(lm.boxcox, new.data.ols, se.fit=TRUE, conf.int=0.95)


est.mean.boxcox <- cbind(box.cox.trans.inverse(est.lm.boxcox.t$linear.predictors, lambda),
                         box.cox.trans.inverse(est.lm.boxcox.t$lower, lambda),
                         box.cox.trans.inverse(est.lm.boxcox.t$upper, lambda))

X <-  cbind(1, new.data2)
pred <-drop(X%*%rq.mod$coef)
V <- summary(rq.mod, cov = TRUE)
df <- V$rdf
tfrac <- qt((1 - 0.95)/2, df)
sdpred <- sqrt(diag(X %*% V$cov %*% t(X)))

est.median.rq <- cbind(pred, pred + tfrac * sdpred %o% 
                         c(1, -1))
colnames(est.median.rq) <- c("fit", "lower", "higher")



est.median.orm.logit <- quantile.orm(mod=orm.logit,new.data=new.data2, probs=0.5,se=TRUE )


test <- predict(lrm.500, new.data.ols, se.fit=TRUE)
est.cdf.glm.logit <- cbind(plogis(test$linear.predictors), plogis(with(test, linear.predictors + 1.96*cbind(-se.fit,se.fit))))


test <- predict(lrm.350, new.data.ols, se.fit=TRUE)
est.cdf.glm.logit2 <- cbind(plogis(test$linear.predictors), plogis(with(test, linear.predictors + 1.96*cbind(-se.fit,se.fit))))


est.cdf.orm.logit <- cdf.orm(mod=orm.logit, new.data=new.data2, at.y=500)
est.cdf.orm.logit2 <- cdf.orm(mod=orm.logit, new.data=new.data2, at.y=350)




plot(1:4,est.mean.orm.logit[,1], type="n",
     axes=FALSE, ylab="CD4", xlab="", xlim=c(0.5, 4.5), ylim=c(200, 400), main="")
#abline(v=1)
axis(2, at=c(200, 250,300,350,400), labels=c("200","250","300", "350","400"))
segments(1:4,est.mean.orm.logit[,3], 1:4, est.mean.orm.logit[,4])
points(1:4,est.mean.orm.logit[,1], pch="-", cex=1)

segments((1:4)+0.2,est.mean.boxcox[,2], (1:4)+0.2, est.mean.boxcox[,3], col=2)
points((1:4)+0.2,est.mean.boxcox[,1], pch="-", cex=1, col=2)
#segments(hernan.700.sw[,2], cd4+3, hernan.700.sw[,3], cd4+3, col=2)
#points(hernan.700.sw[,1], cd4+3, pch="|", cex=0.4, col=2)
legend("top", c("orm (logit)", "Box-Cox"), col=c(1,2), lty=c(1,1),bty="n")
axis(1, 1:4, labels=c("BPI","NNRTI","UBPI","Other"),
     las=1, tck=-0.01, col=1, cex.axis=0.8)
box()


plot(1:4,est.median.orm.logit$quantile, type="n",
     axes=FALSE, ylab="CD4", xlab="", xlim=c(0.5, 4.5), ylim=c(200, 400), main="")
#abline(v=1)
axis(2, at=c(200,250,300,350,400), labels=c("200","250","300", "350","400"))
segments(1:4,est.median.orm.logit$lb, 1:4, est.median.orm.logit$ub)
points(1:4,est.median.orm.logit$quantile, pch="-", cex=1)

segments((1:4)+0.2,est.median.rq[,2], (1:4)+0.2, est.median.rq[,3], col=2)
points((1:4)+0.2,est.median.rq[,1], pch="-", cex=1, col=2)
#segments(hernan.700.sw[,2], cd4+3, hernan.700.sw[,3], cd4+3, col=2)
#points(hernan.700.sw[,1], cd4+3, pch="|", cex=0.4, col=2)
legend("top", c("orm (logit)", "Median regression"), col=c(1,2), lty=c(1,1),bty="n")
axis(1, 1:4, labels=c("BPI","NNRTI","UBPI","Other"),
     las=1, tck=-0.01, col=1, cex.axis=0.8)
box()

plot(1:4,1-est.cdf.orm.logit2$est, type="n",
     axes=FALSE, ylab="probability", xlab="", xlim=c(0.5, 4.5), ylim=c(0, 0.6), main="")

axis(2, at=c(0,0.1,0.2,0.3,0.4, 0.5, 0.6), labels=c("0.0","0.1","0.2","0.3","0.4", "0.5","0.6"))
segments(1:4,1-est.cdf.orm.logit2$lb, 1:4, 1-est.cdf.orm.logit2$ub)
points(1:4,1-est.cdf.orm.logit2$est, pch="-", cex=1)

segments((1:4)+0.2,est.cdf.glm.logit2[,2], (1:4)+0.2, est.cdf.glm.logit2[,3], col=2)
points((1:4)+0.2,est.cdf.glm.logit2[,1], pch="-", cex=1, col=2)

legend("top", c("orm (logit)", "Logistic regression"), col=c(1,2), lty=c(1,1),bty="n")
axis(1, 1:4, labels=c("BPI","NNRTI","UBPI","Other"),
     las=1, tck=-0.01, col=1, cex.axis=0.8)
box()

plot(1:4,1-est.cdf.orm.logit$est, type="n",
     axes=FALSE, ylab="probability", xlab="", xlim=c(0.5, 4.5), ylim=c(0, 0.6), main="")
axis(2, at=c( 0, 0.1, 0.2, 0.3, 0.4), labels=c("0.0", "0.1", "0.2","0.3", "0.4", "0.5", "0.6"))
segments(1:4,1-est.cdf.orm.logit$lb, 1:4, 1-est.cdf.orm.logit$ub)
points(1:4,1-est.cdf.orm.logit$est, pch="-", cex=1)
segments((1:4)+0.2,est.cdf.glm.logit[,2], (1:4)+0.2, est.cdf.glm.logit[,3], col=2)
points((1:4)+0.2,est.cdf.glm.logit[,1], pch="-", cex=1, col=2)
legend("top", c("orm (logit)", "Logistic regression"), col=c(1,2), lty=c(1,1),bty="n")
axis(1, 1:4, labels=c("BPI","NNRTI","UBPI","Other"),
     las=1, tck=-0.01, col=1, cex.axis=0.8)
box()

dev.off()





